#include "Matrix.h"

#include <string>
#include <stdexcept>

template<typename T, std::size_t M, std::size_t N>
Matrix<T, M, N>::Matrix(T value)
{
    mData = std::vector<T>(M * N, value);
}

template<typename T, std::size_t M, std::size_t N>
Matrix<T, M, N>::Matrix(T value, InputType inputType)
{
    if (inputType == InputType::DIAGONAL)
    {
        mData = std::vector<T>(M * N, 0.0);
        setDiagonals(value);
    }
    else
    {
        mData = std::vector<T>(M * N, value);
    }
}

template<typename T, std::size_t M, std::size_t N>
void Matrix<T, M, N>::applyTo(Vector<T, N>&) const
{
 
}

template<typename T, std::size_t M, std::size_t N>
void Matrix<T, M, N>::setDiagonals(T value)
{
    if (!isSquare())
    {
        throw std::logic_error("Requested setting diagonals on non-square Matrix.");
    }

    for (std::size_t idx = 0; idx < M; idx++)
    {
        mData[getFlatIndex(idx, idx)] = value;
    }
}

template<typename T, std::size_t M, std::size_t N>
void Matrix<T, M, N>::setDiagonals(const std::vector<T>& values)
{
    if (!isSquare())
    {
        throw std::logic_error("Requested setting diagonals on non-square Matrix.");
    }

    for (std::size_t idx = 0; idx < values.size(); idx++)
    {
        if (idx >= M)
        {
            break;
        }
        mData[getFlatIndex(idx, idx)] = values[idx];
    }
}

template<typename T, std::size_t M, std::size_t N>
void Matrix<T, M, N>::setItem(std::size_t rowId, std::size_t columnId, T value)
{
    const auto index = getFlatIndex(rowId, columnId);
    if (index >= mData.size())
    {
        throw std::range_error("Out of bounds array access: " + std::to_string(index) + " for size " + std::to_string(mData.size()));
    }
    mData[index] = value;
}

template<typename T, std::size_t M, std::size_t N>
bool Matrix<T, M, N>::isIdentity() const
{
    if (!isSquare())
    {
        return false;
    }

    for(std::size_t idx=0; idx<M; idx++)
    {
        for(std::size_t jdx=0; jdx<N; jdx++)
        {
            if (idx == jdx)
            {
                if (getItem(idx, jdx) != 1.0)
                {
                    return false;
                }
            }
            else
            {
                if (getItem(idx, jdx) != 0.0)
                {
                    return false;
                }
            }
        }
    }
    return true;
}

template<typename T, std::size_t M, std::size_t N>
T Matrix<T, M, N>::getItem(std::size_t rowId, std::size_t columnId) const
{
    const auto index = getFlatIndex(rowId, columnId);
    if (index >= mData.size())
    {
        throw std::range_error("Out of bounds array access: " + std::to_string(index) + " for size " + std::to_string(mData.size()));
    }
    return mData[getFlatIndex(rowId, columnId)];
}

template<typename T, std::size_t M, std::size_t N>
bool Matrix<T, M, N>::isSquare() const
{
    return M == N;
}

template<typename T, std::size_t M, std::size_t N>
bool Matrix<T, M, N>::isEqual(const Matrix<T, M, N>& matrix) const
{
    return mData == matrix.mData;
}

template<typename T, std::size_t M, std::size_t N>
std::size_t Matrix<T, M, N>::getFlatIndex(std::size_t rowId, std::size_t columnId) const
{
    return columnId + rowId*N;
}

template class Matrix<double, 2, 2>;
template class Matrix<double, 3, 3>;
template class Matrix<double, 4, 4>;

template class Matrix<std::complex<double>, 2, 2>;
template class Matrix<std::complex<double>, 3, 3>;
