#include "Line.h"

template<std::size_t DIM>
Line<DIM>::Line(const PointCollection<DIM>& points)
		: mPoints(points)
{

}

template<std::size_t DIM>
Line<DIM>::Line(InputBufferType bufferType, const std::vector<double>& buffer)
{
	if (bufferType == InputBufferType::XY)
	{
		for (std::size_t idx = 0; idx < buffer.size(); idx += 2)
		{
			const auto x = buffer[idx];
			const auto y = buffer[idx + 1];
			mPoints.addPoint(Point<DIM>(x, y));
		}
	}
	else if (bufferType == InputBufferType::HORIZONTAL)
	{
		for (std::size_t idx = 0; idx < buffer.size(); idx ++)
		{
			mPoints.addPoint(Point<DIM>(buffer[idx], 0.0));
		}
	}
	else if (bufferType == InputBufferType::VERTICAL)
	{
		for (std::size_t idx = 0; idx < buffer.size(); idx++)
		{
			mPoints.addPoint(Point<DIM>(0.0, buffer[idx]));
		}
	}
}

template<std::size_t DIM>
Vector<double, DIM> Line<DIM>::getEndOffset() const
{
	return mPoints.getEndPoint().getOriginOffset();
}

template<std::size_t DIM>
const PointCollection<DIM>& Line<DIM>::getPoints() const
{
	return mPoints;
}

template<std::size_t DIM>
AbstractGeometricItem::Type Line<DIM>::getType() const
{
	return AbstractGeometricItem::Type::LINE;
}

template<std::size_t DIM>
Bounds Line<DIM>::getBounds() const
{
	auto bounds = mPoints.getBounds();
	bounds.includePoint(0.0, 0.0, 0.0);
	return bounds;
}

template class Line<2>;
template class Line<3>;

