#include "LineSegment.h"

template<std::size_t DIM>
LineSegment<DIM>::LineSegment(const Point<DIM>& p0, const Point<DIM>& p1)
    : mP0(p0),
      mP1(p1)
{

}

template<std::size_t DIM>
std::unique_ptr<LineSegment<DIM> > LineSegment<DIM> ::Create(const Point<DIM>& p0, const Point<DIM>& p1)
{
    return std::make_unique<LineSegment<DIM> >(p0, p1);
}

template<std::size_t DIM>
double LineSegment<DIM> ::getLength() const
{
    return mP0.getDistance(mP1);
}

template<std::size_t DIM>
const Point<DIM>& LineSegment<DIM> ::getPoint0() const
{
    return mP0;
}

template<std::size_t DIM>
const Point<DIM>& LineSegment<DIM> ::getPoint1() const
{
    return mP1;
}

template<std::size_t DIM>
bool LineSegment<DIM>::isHorizontal() const
{
	return mP0.getDeltaY(mP1) == 0.0;
}

template<std::size_t DIM>
bool LineSegment<DIM>::isVertical() const
{
	return mP0.getDeltaX(mP1) == 0.0;
}

template<std::size_t DIM>
Bounds LineSegment<DIM>::getBounds() const
{
    const auto minX = std::min(mP0.getX(), mP1.getX());
    const auto maxX = std::max(mP0.getX(), mP1.getX());

    const auto minY = std::min(mP0.getY(), mP1.getY());
    const auto maxY = std::max(mP0.getY(), mP1.getY());

    return {minX, maxX, minY, maxY};
}

template<std::size_t DIM>
Vector<double, DIM> LineSegment<DIM>::getEndOffset() const
{
    return getPoint1().getOriginOffset();
}

template<std::size_t DIM>
AbstractGeometricItem::Type LineSegment<DIM>::getType() const
{
    return AbstractGeometricItem::Type::LINE_SEGMENT;
}

template<std::size_t DIM>
Vector<double, DIM> LineSegment<DIM>::asVector() const
{
    return mP1.getDelta(mP0);
}

template class LineSegment<2>;
template class LineSegment<3>;