#include "MainApplication.h"
#include "FileLogger.h"
#include "File.h"
#include "DocumentConverter.h"
#include "DrawingSurface.h"
#include "DrawingContext.h"
#include "Grid.h"
#include "Image.h"
#include "TriMesh.h"
#include "AbstractPainter.h"
#include "TextNode.h"
#include "DiscretePoint.h"
#include "Scene.h"

#include "BasicWebApp.h"


#include <filesystem>

MainApplication::MainApplication()
     :  mCommandLineArgs(),
        mDatabaseManager()

{

}

std::unique_ptr<MainApplication> MainApplication::Create()
{
    return std::make_unique<MainApplication>();
}


MainApplication::~MainApplication()
{

}

void MainApplication::initialize(CommandLineArgsUPtr commandLineArgs, std::unique_ptr<IApplicationContext> applicationContext)
{
    if (applicationContext)
    {
        mApplicationContext = std::move(applicationContext);
    }
    
    mCommandLineArgs = std::move(commandLineArgs);
    const auto launch_path = mCommandLineArgs->getLaunchPath().string();

    FileLogger::GetInstance().SetWorkDirectory(launch_path);
    FileLogger::GetInstance().Open();
    MLOG_INFO("Launched");

    if (false)
    {
        mDatabaseManager = DatabaseManager::Create();
        mDatabaseManager->openDatabase(launch_path + "/database.db");
        MLOG_INFO("Created DB");

        mNetworkManager = NetworkManager::Create();
        MLOG_INFO("Created Network Manager");

        mAudioManager = AudioManager::Create();
        MLOG_INFO("Created Audio Manager");
    }
}

void MainApplication::run()
{
    std::string program_type;
    if(mCommandLineArgs->getArgs().size() > 1)
    {
        program_type = mCommandLineArgs->getArgs()[1];
    }

    std::string input_path;
    std::string output_path;
    for(unsigned idx=1; idx< mCommandLineArgs->getArgs().size(); idx++)
    {
        auto arg = mCommandLineArgs->getArgs()[idx];
        if(arg == "-input" && mCommandLineArgs->getArgs().size() > idx+1)
        {
            input_path = mCommandLineArgs->getArgs()[idx + 1];
        }
        else if(arg == "-output" && mCommandLineArgs->getArgs().size() > idx+1)
        {
            output_path = mCommandLineArgs->getArgs()[idx + 1];
        }
    }

    if(program_type == "-server")
    {
        runServer();
    }
    else if(program_type == "-audio")
    {
        playAudio();
    }
    else if(program_type == "-convert")
    {
        convertDocument(input_path, output_path);
    }
    else if(program_type == "-draw")
    {
        /*
        auto drawing_manager = DrawingManager::Create();
        drawing_manager->InitalizeSurface(400, 400);

        auto text = TextNode::Create("Hello World", DiscretePoint(20, 20));
        drawing_manager->AddText(text.get());
        drawing_manager->RenderToFile("test.png");
        */
    }
    else
    {
        MLOG_ERROR("Unknown program type: " + program_type);
    }

    shutDown();
}

void MainApplication::runServer()
{
    BasicWebApp web_app;
    mNetworkManager->runHttpServer(&web_app);
}

void MainApplication::playAudio()
{
    //MidiReader reader;
    //reader.Read("/home/james/sample.mid");
    //mAudioManager->Play();
}

void MainApplication::convertDocument(const std::string& inputPath, const std::string& outputPath)
{
    auto input_file = File(std::filesystem::path(inputPath));
    auto output_file = File(std::filesystem::path(outputPath));
    MLOG_INFO("Converting: " + inputPath + " to " + outputPath);
    DocumentConverter converter;
    converter.convert(&input_file, &output_file);
}

CommandLineArgs* MainApplication::getCommandLineArgs() const
{
    return mCommandLineArgs.get();
}

void MainApplication::shutDown()
{
    if (mDatabaseManager)
    {
        mDatabaseManager->onShutDown();
    }
    
    MLOG_INFO("Shut down");
    FileLogger::GetInstance().Close();
}
