#include "CircleNode.h"

#include "SceneModel.h"
#include "SceneInfo.h"

#include "Circle.h"

CircleNode::CircleNode(const Transform& transform, double radius)
:    GeometryNode(transform),
     mRadius(radius)
{
    mMinorRadius = mRadius;
}

CircleNode::~CircleNode()
{

}

CircleNode::Type CircleNode::getType()
{
    return Type::Circle;
}

double CircleNode::getRadius() const
{
    return mRadius;
}

void CircleNode::setRadius(double radius)
{
    if (mRadius != radius)
    {
        mRadius = radius;
        mGeometryIsDirty = true;
    }
}

void CircleNode::setMinorRadius(double radius)
{
    if (mMinorRadius != radius)
    {
        mMinorRadius = radius;
        mGeometryIsDirty = true;
    }
}

void CircleNode::createOrUpdateGeometry(SceneInfo*)
{
    auto circle = std::make_unique<Circle>(mRadius);
    circle->setMinorRadius(mMinorRadius);

    if (!mBackgroundItem)
    {
        mBackgroundItem = std::make_unique<SceneModel>(std::move(circle));
        mBackgroundItem->setName(mName + "_Model");
    }
    else
    {
        mBackgroundItem->updateGeometry(std::move(circle));
    }
}
